@@ -1,6 +1,10 @@
 /*
  *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
+ *
+ *	Port of NDBM version to GDBM by Matti Aarnio 1989-1994
+ *
+ *	Currently uses GDBM version 1.7.3 !
  */
 
 /* LINTLIBRARY */
@@ -8,6 +12,24 @@
 #include "mailer.h"
 #ifdef	USE_GDBM
 #include <gdbm.h>
+#ifndef	GDBM_VERSION /* In a hope of GDBM 1.7.4 having this #define, and
+			the   gdbm_filefno() function.. */
+	/* Because GDBM 1.7.3 does not have gdbm_filefno() or some such,
+	   we must do "Horrible Kludges"(TM) to find the fileno..	*/
+	/* THIS MUST TRACK THE GDBM INTERNAL DATA STRUCTURE ! Auurgh...! */
+struct __gdbmfoo {
+	char *name;     /* Space alloc for things that are there */
+	int read_write;
+#ifdef GDBM_FASTMODE	/* This appeared somewhen in between 0.9 and 1.7.3 */
+        int fast_write;
+#endif
+	void (*fatal_err)();
+	int desc;       /* this is what we do need! */
+	/* the rest is irrelevant for us.. */
+};
+#define gdbm_filefno(db) (((struct __gdbmfoo *) (db))->desc)
+#endif
+
 #include <fcntl.h>
 #include <sys/file.h>
 #include "search.h"
@@ -15,14 +37,7 @@
 
 /* extern int gdbm_errno; */
 extern int deferit;
-
-/* XXX: THIS IS A SERIOUS KLUDGE! */
-struct gdbmfoo {	/* extract from gdbmdefs.h - gdbm sources */
-	char *name;	/* Valid for GDBM 0.9beta! */
-	int read_write;
-	void (*fatal_err)();
-	int desc;	/* open() handle for file */
-};
+extern int nobody;	/* UID of NOBODY */
 
 
 /*
@@ -62,15 +77,18 @@ reopen:
 	key.dptr = (char *)(sip->key);
 	key.dsize = strlen((char *)(sip->key)) + 1;
 	val = gdbm_fetch(db, key);
-	gdbm_close(db);
 	if (val.dptr == NULL) {
 		if (!retry && gdbm_errno) {
+			gdbm_close(db);
+			sp_install(symid, (u_char *)NULL, 0, spt_files);
 			++retry;
 			goto reopen;
 		}
 		return NULL;
 	}
-	return newstring((u_char *)strnsave(val.dptr, val.dsize));
+	tmp = newstring((u_char *)strnsave(val.dptr, val.dsize));
+	free(val.dptr);
+	return tmp;
 }
 
 /*
@@ -180,8 +198,8 @@ remove_gdbm(sip)
 	if (gdbm_delete(db, key) < 0) {
 		++deferit;
 		v_set(DEFER, DEFER_IO_ERROR);
-		fprintf(stderr, "remove_gdbm: cannot remove \"%s\"\n",
-				sip->key);
+		fprintf(stderr, "remove_gdbm: cannot remove \"%s\" from \"%s\"\n",
+				sip->key, sip->file);
 		return EOF;
 	}
 	return 0;
@@ -197,28 +215,33 @@ print_gdbm(sip, outfp)
 	FILE *outfp;
 {
 	GDBM_FILE db;
-	datum key, val;
+	datum key, nextkey, val;
 
 	if ((db = open_gdbm(sip, O_RDONLY, "print_gdbm", NULL)) == NULL)
 		return;
-	for (key = gdbm_firstkey(db); key.dptr != NULL; key = gdbm_nextkey(db, val)) {
+	for (key = gdbm_firstkey(db); key.dptr != NULL; nextkey = gdbm_nextkey(db, key)) {
 		val = gdbm_fetch(db, key);
 		if (val.dptr == NULL)
 			continue;
 		if (gdbm_errno)
 			break;
-		if (*val.dptr == '\0')
+		if (val.dptr == NULL || *val.dptr == '\0')
 			fprintf(outfp, "%s\n", key.dptr);
 		else
 			fprintf(outfp, "%s\t%s\n", key.dptr, val.dptr);
+		if (val.dptr != NULL)
+			free(val.dptr);
+		free(key.dptr);
+		key = nextkey;
 	}
 	fflush(outfp);
 }
 
 /*
- * Print the uid of the owner of the database.  Note that for gdbm-style
- * databases there are several files involved so picking one of them for
- * security purposes is very dubious.
+ * Print the uid of the owner of the database file.
+ * (For the security purposes.)  Unlike  DBM and NDBM,
+ * for GDBM there is only one file, and this makes
+ * more sense..
  */
 
 void
@@ -232,10 +255,11 @@ owner_gdbm(sip, outfp)
 
 	if ((db = open_gdbm(sip, O_RDONLY, "owner_gdbm")) == NULL)
 		return;
-	fno = ((struct gdbmfoo *) db)->desc;
+	fno = gdbm_filefno(db);
 	if (fstat(fno, &stbuf) < 0) {
 		fprintf(stderr, "owner_gdbm: cannot fstat(\"%s\")!\n",
 				sip->file);
+		fprintf(outfp, "%d\n", nobody);
 		return;
 	}
 	fprintf(outfp, "%d\n", stbuf.st_uid);
@@ -247,12 +271,6 @@ modp_gdbm(sip)
 	struct search_info *sip;
 {
 	GDBM_FILE db;
-	struct gdbmfoo {	/* extract from gdbmdefs.h - gdbm sources */
-		char *name;	/* Valid for GDBM 0.9beta! */
-		int read_write;
-		void (*fatal_err)();
-		int desc;	/* open() handle for file */
-	} gdbmfoo;	
 	int	fno;
 	struct stat stbuf;
 	struct spblk *spl;
@@ -261,7 +279,7 @@ modp_gdbm(sip)
 	if (sip->file == NULL
 	    || (db = open_gdbm(sip, O_RDONLY, "modp_gdbm")) == NULL)
 		return 0;
-	fno = ((struct gdbmfoo *) db)->desc;
+	fno = gdbm_filefno(db);
 	if (fstat(fno, &stbuf) < 0) {
 		fprintf(stderr, "modp_gdbm: cannot fstat(\"%s\")!\n",
 				sip->file);
@@ -272,7 +290,8 @@ modp_gdbm(sip)
 	spl = sp_lookup(symid, spt_modcheck);
 	if (spl != NULL) {
 		rval = stbuf.st_mtime != (time_t)spl->data
-			|| stbuf.st_nlink != (int)spl->mark;
+			|| stbuf.st_nlink != (int)spl->mark
+			|| stbuf.st_nlink == 0;
 	} else
 		rval = 0;
 	sp_install(symid, (u_char *)stbuf.st_mtime,
