@@ -43,41 +43,6 @@ char *Copyright2= "Copyright 1991-1994 Matti Aarnio";
 #ifdef FCNTL_H
 # include FCNTL_H
 #endif
-#ifdef	USE_STATVFS
-# include <sys/statvfs.h>
-# define FSTATFS fstatvfs /* Existence of this means (usually) SYSV code */
-# define STATFSTYPE struct statvfs
-#else	/* !USE_STATVFS */
-#ifdef	SYSV
-# include <sys/statfs.h>
-# define FSTATFS fstatfs /* Existence of this means (usually) SYSV code */
-# define STATFSTYPE struct statfs
-#else /* Hmm.. Assuming it appears on all NON-SysV systems.. */
-#ifdef ULTRIX
-	/* From: James S MacKinnon <jmack@Phys.UAlberta.Ca> */
-# include <sys/types.h>
-# include <sys/stat.h>
-# include <sys/param.h>
-# include <sys/mount.h>
-# include <netinet/in.h>
-# include <nfs/nfs_clnt.h>
-# include <nfs/vfs.h>
-# define FSTATFS statfs	/* Existence of this means (usually) SYSV code */
-# define STATFSTYPE struct statfs
-#else /* !ULTRIX */
-#ifdef bsdi
-# include <sys/types.h>
-# include <sys/mount.h>
-/* # define FSTATFS fstatfs */ /* nonexistence of this means BSD code */
-# define STATFSTYPE struct statfs
-#else /* !bsdi */
-# include <sys/vfs.h>
-/* # define FSTATFS fstatfs */ /* nonexistence of this means BSD code */
-# define STATFSTYPE struct statfs
-#endif /* !bsdi */
-#endif /* !ULTRIX */
-#endif /* !SYSV */
-#endif /* !USE_STATVFS */
 
 #ifdef	USE_UNIONWAIT
 #include <sys/wait.h>
@@ -798,11 +763,14 @@ char *cp;
 	  type(carp->cmd==Hello2 ? -250: 250,
 	       " %s Hello %s", myhostname, helobuf);
 	if (carp->cmd == Hello2) {
-		if (mfp == NULL &&
-		    (mfp = mail_open(MSG_RFC822)) == NULL) {
-			type(452, (char *)NULL);
-			return;
-		}
+		/*  -- this is NOT useful, while we do not report
+		       available disk space on the ESMTP initial greeting..
+		   if (mfp == NULL &&
+		   (mfp = mail_open(MSG_RFC822)) == NULL) {
+		   type(452, (char *)NULL);
+		   return;
+		   }
+		  */
 
 		/* ESMTP -- RFC 1651,1652,1653,1428 thingies */
 		type(-250,"SIZE");
@@ -824,7 +792,6 @@ int insecure;
 	char *s;
 	int rc;
 	char *drpt_envid;
-	STATFSTYPE statbuf;	/* SysV and BSD definitions differ..	*/
 
 	/* For ESMTP SIZE-option use we need to know
 	   how much space we have, it is easiest by
@@ -839,22 +806,7 @@ int insecure;
 		type(452, (char *)NULL);
 		return;
 	}
-#if defined(__linux__)
-	if ((rc = fstatfs(fileno(mfp), &statbuf)) == 0) {
-	  availspace = statbuf.f_bavail * statbuf.f_bsize;
-	}
-#else
-#if defined(FSTATFS)
-	if ((rc = FSTATFS(fileno(mfp), &statbuf)) == 0) {
-	  availspace = statbuf.f_bavail * statbuf.f_frsize;
-	}
-#else /* no __linux__, no FSTATFS */
-	if ((rc = fstatfs(fileno(mfp),
-			  &statbuf,sizeof statbuf,0)) == 0){
-	  availspace = statbuf.f_bfree * statbuf.f_bsize;
-	}
-#endif
-#endif
+	availspace = fd_statfs(fileno(mfp));
 	if (availspace < 0)
 	  availspace = 2000000000; /* Over 2G ? */
 	availspace >>= 1;
