@@ -22,6 +22,7 @@
 #include <sysexits.h>
 #include <varargs.h>
 #include <sys/param.h>
+#include <syslog.h>
 #include <fcntl.h>
 #include <sys/file.h>
 #include <sys/stat.h>
@@ -611,9 +612,28 @@ deliver(dp, rp, timestring)
 	    if (rp->status != EX_OK)
 	      return;
 	  }
-	  if (*maild == 0)	/* didn't find it? */
-	    if (!creatembox(rp, &file, &st.st_uid, &st.st_gid, pw))
-	      return;		/* creatembox sets status */
+	  if (*maild == 0) {	/* didn't find it? */
+	    /* if (!creatembox(rp, &file, &st.st_uid, &st.st_gid, pw)) */
+
+	    char *mailbox = getzenv("MAILBOX");
+	    notaryreport(rp->addr->user,"failed",
+			 "466 (System mailbox configuration is wrong!  No such directory!  Aargh!)");
+	    DIAGNOSTIC(rp, EX_TEMPFAIL,
+		       "System mailbox configuration is wrong!  No such directory (%s)!  Aargh!",maildirs[0]);
+#ifdef  LOG_PID
+#ifdef  LOG_MAIL
+	    openlog("scheduler", LOG_PID, LOG_MAIL);
+#else   /* !LOG_MAIL */
+	    openlog("scheduler", LOG_PID);
+#endif  /* LOG_MAIL */
+#endif  /* LOG_PID */
+	    if (!mailbox) 
+	      syslog(LOG_ALERT, "ZMailer mailbox can't open local mail folder directory! (Using BUILTIN list of choices)\n");
+	    else
+	      syslog(LOG_ALERT, "ZMailer mailbox can't open local mail folder directory!  $MAILBOX='%s'\n", mailbox);
+	    closelog();
+	    return;	/* creatembox sets status */
+	  }
 #if	defined(BIFF) || defined(RBIFF)
 	  if (!dobiff && !dorbiff)
 	    break;
@@ -1436,6 +1456,7 @@ appendlet(dp, rp, fp, file, ismime)
 	register int bufferfull;
 	char iobuf[BUFSIZ];
 	FILE *mfp;
+	int mfd = dp->msgfd;
 
 	if (ismime)
 	  writemimebuf((FILE *)NULL, (char *)NULL, 0);	/* initialize state */
