@@ -259,7 +259,7 @@ extern int cistrcmp(), cistrncmp();
 extern void getdaemon();
 
 FILE *verboselog = NULL;
-
+int smtp_isopen = 0;
 
 int
 main(argc, argv)
@@ -275,6 +275,9 @@ main(argc, argv)
 	int checkmx = 0;	/* check all destination hosts for MXness */
 #endif	/* BIND */
 
+	smtpfp[0] = NULL;
+	smtpfp[1] = NULL;
+
 	pid = getpid();
 	ipaddress[0] = '\0';
 	msgfile = "?";
@@ -411,9 +414,15 @@ main(argc, argv)
 
 	/* defer opening a connection until we know there is work */
 	smtpstatus = EX_DATAERR;
+	smtp_isopen = 0;
+
 	while (!getout && fgets(file, sizeof file, stdin)) {
 	  if (emptyline(file, sizeof file))
 	    break;
+
+	  /* if(logfp) fprintf(logfp,
+	     "%d#\t(fdcnt=%d, file:%s)\n",pid,countfds(),file); */
+
 #ifdef	BIND
 	  if (checkmx)
 	    dp = ctlopen(file, channel, host, &getout, rightmx, matchroutermxes);
@@ -431,12 +440,15 @@ main(argc, argv)
 	    if (verboselog)
 	      setbuf(verboselog,NULL);
 	  }
-	  if (smtpstatus == EX_DATAERR) {
+	  if (!smtp_isopen) {
 	    if ((first_uid = atoi(dp->senders->misc)) < 0 ||
 		first_uid == nobody)
 	      first_uid = daemon_uid;
 	    smtpstatus = smtpopen(host, smtpfp, verboselog);
+	    if (smtpstatus == EX_OK)
+	      smtp_isopen = 1;
 	  }
+
 	  smtpstatus = process(dp, smtpstatus, host, smtpfp, verboselog);
 	  ctlclose(dp);
 	}
@@ -444,11 +456,14 @@ main(argc, argv)
 	  smtpstatus = smtpwrite(smtpfp, verboselog, 0, "QUIT");
 	  fclose(smtpfp[0]);
 	  fclose(smtpfp[1]);
+	  smtp_isopen = 0;
 	}
 	if (verboselog != NULL)
 		fclose(verboselog);
 	if (logfp != NULL)
 		fclose(logfp);
+	if (smtpstatus = EX_DATAERR)
+		smtpstatus = EX_OK; /* Nothing processed ?? */
 	exit(smtpstatus);
 	/* NOTREACHED */
 	return smtpstatus;
@@ -482,14 +497,19 @@ process(dp, smtpstatus, host, smtpfp, verboselog)
 			dp->logident);
 		fflush(logfp);
 	      }
-	      smtpstatus = deliver(dp, smtpfp, verboselog, rphead, rp->next);
-	      if (smtpstatus != EX_OK) {
-		fclose(smtpfp[0]);
-		fclose(smtpfp[1]);
+	      if (!smtp_isopen) {
 		if ((first_uid = atoi(dp->senders->misc)) < 0 ||
 		    first_uid == nobody)
 		  first_uid = daemon_uid;
 		smtpstatus = smtpopen(host, smtpfp, verboselog);
+		if (smtpstatus == EX_OK)
+		  smtp_isopen = 1;
+	      }
+	      smtpstatus = deliver(dp, smtpfp, verboselog, rphead, rp->next);
+	      if (smtpstatus != EX_OK) {
+		fclose(smtpfp[0]); smtpfp[0] = NULL;
+		fclose(smtpfp[1]); smtpfp[1] = NULL;
+		smtp_isopen = 0;
 	      }
 	      rphead = rp->next;
 	    } else {
@@ -776,13 +796,14 @@ appendlet(dp, fp, verboselog, hsize, msize, convertmode)
 	/* can we use cache of message body data */
 	if (convertmode == _CONVERT_NONE &&
 	    readalready != 0) {
-	  if (writebuf(fp, let_buffer, readalready) != readalready)
+	  if (writebuf(fp, let_buffer, readalready) != readalready) {
 	    if (gotalarm) {
 	      sprintf(remotemsg,"500 (let_buffer write timeout!  DATA %d/%d [%d%%])",
 		      hsize, msize, (hsize*100+msize/2)/msize);
 	      /* return EX_IOERR; */
 	    }
 	    return EX_IOERR;
+	  }
 	  if (let_buffer[readalready-1] != '\n')
 	    writebuf(fp, "\n", 1);
 	  if (statusreport)
@@ -944,7 +965,7 @@ writebuf(fp, buf, len)
 	}
 	state  = save;
 	for (cp = buf, n = len; n > 0; --n, ++cp) {
-	  int c = *(unsigned char *)cp;
+	  unsigned char c = *(unsigned char *)cp;
 
 	  if (state && c != '\n') {
 	    state = 0;
@@ -1431,6 +1452,10 @@ makeconn(hp, fdp, verboselog, fparr, mxcnt)
 	  }
 	  switch (vcsetup(*hp_getaddr(), hp->h_length, fdp, hostbuf, verboselog)) {
 	  case EX_OK:
+	    /* Clean (close) these fds -- they have noted to leak.. */
+	    if (fparr[0] != NULL) fclose(fparr[0]);
+	    if (fparr[1] != NULL) fclose(fparr[1]);
+
 	    fparr[0] = (FILE*) fdopen(*fdp, "r");
 	    fparr[1] = (FILE*) fdopen(*fdp, "w");
 	    if (esmtp_on_banner > 0)
@@ -2434,6 +2459,7 @@ struct ctldesc *dp;
 	register int bufferfull;
 	int lastwasnl;
 	long mfd_pos;
+	int mfd = dp->msgfd;
 
 	/* can we use cache of message body data ? */
 	if (readalready != 0) {
@@ -2445,7 +2471,7 @@ struct ctldesc *dp;
 	/* we are assumed to be positioned properly at start of message body */
 	bufferfull = 0;
 
-	mfd_pos = lseek(mfd, 0L, SEEK_CUR);
+	mfd_pos = lseek(mfd, dp->msgbodyoffset, SEEK_SET);
 	
 	while ((i = read(mfd, let_buffer, sizeof let_buffer)) != 0) {
 	  if (i < 0) {
@@ -2517,3 +2543,19 @@ void getdaemon()
 	if (!pw) daemon_uid = 0; /* Let it be root, if nothing else */
 	else     daemon_uid = pw->pw_uid;
 }
+
+#if 0
+/* Debugging stuff.. */
+int countfds()
+{
+  int fds = getdtablesize();
+  int i;
+  int cnt = 0;
+
+  for (i=0; i<fds; ++i)
+    if (fcntl(i,F_GETFL) >= 0)
+      ++cnt;
+
+  return cnt;
+}
+#endif
